﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Collections.Concurrent;
using Examples.Bing.Cloud.Restaurants.BingRoute;
using System.Diagnostics;

namespace Examples.Bing.Cloud.Restaurants
{
    public class RestaurantLocator : IRestaurantLocator
    {
        private static List<Restaurant> restaurants;

        public static string BingApiKey = "AuISdif3bCj44Jn7bqCxMRwSOVDJ9gY3IKnsExkrao57c6sCQh_GxYugm87qRYfh";
        
        public RestaurantLocator()
        {
            if (restaurants == null)
            {
                restaurants = new List<Restaurant>();
                restaurants.Add(new Restaurant("My Restaurant", "Curch Street 100", "New York", "10013"));
                restaurants.Add(new Restaurant("Fish Restaurant", "46th Street 52", "New York", "10017"));
                restaurants.Add(new Restaurant("Hamburger Restaurant", "Second Avenue 20", "New York", "10003"));
                restaurants.Add(new Restaurant("Some Restaurant", "Broadway 2350", "New York", "10024 "));
                restaurants.Add(new Restaurant("Good Restaurant", "43 West 24th Street", "New York", "10010"));
                restaurants.Add(new Restaurant("Popular Restaurant", "40 West 40th Street", "New York", "10018"));
                restaurants.Add(new Restaurant("A Restaurant", "509 E. 6th Street", "New York", "10009"));
            }
        }

        public NearResult GetNear(BingRoute.Location location)
        {
            var near = new NearResult();
            near.Restaurants = new List<RestaurantResult>();

            var results = new ConcurrentBag<RestaurantResult>();

            // Build geo request.
            var reverseRequest = new BingGeo.ReverseGeocodeRequest();
            reverseRequest.Credentials = new BingGeo.Credentials();
            reverseRequest.Credentials.Token = BingApiKey;
            reverseRequest.Location = new BingGeo.Location();
            reverseRequest.Location.Altitude = location.Altitude;
            reverseRequest.Location.Latitude = location.Latitude;
            reverseRequest.Location.Longitude = location.Longitude;

            // Get the restaurant's location.
            var geoClient = new BingGeo.GeocodeServiceClient();
            var reverseResponse = geoClient.ReverseGeocode(reverseRequest);

            // Get the address.
            if (reverseResponse.Results.Any())
            {
                near.Address = reverseResponse.Results.FirstOrDefault().Address.FormattedAddress;
            }

            Parallel.ForEach(restaurants, (resto) =>
                {
                    try
                    {
                        Trace.WriteLine(DateTime.Now + " - Processing: " + resto.Name);

                        // Build geo request.
                        var geoRequest = new BingGeo.GeocodeRequest();
                        geoRequest.Credentials = new BingGeo.Credentials();
                        geoRequest.Credentials.Token = BingApiKey;
                        geoRequest.Address = new BingGeo.Address();
                        geoRequest.Address.AddressLine = resto.Street;
                        geoRequest.Address.PostalCode = resto.PostalCode;
                        geoRequest.Address.PostalTown = resto.City;

                        // Get the restaurant's location.
                        var geoResponse = geoClient.Geocode(geoRequest);

                        // Restaurant position.
                        if (geoResponse.Results.Any())
                        {
                            var restoLocation = geoResponse.Results.FirstOrDefault().Locations.FirstOrDefault();
                            if (restoLocation != null)
                            {
                                // Build route request.
                                var fromWaypoint = new Waypoint();
                                fromWaypoint.Description = "Current Position";
                                fromWaypoint.Location = new Location();
                                fromWaypoint.Location.Altitude = location.Altitude;
                                fromWaypoint.Location.Latitude = location.Latitude;
                                fromWaypoint.Location.Longitude = location.Longitude;

                                var toWaypoint = new Waypoint();
                                toWaypoint.Description = resto.Name;
                                toWaypoint.Location = new Location();
                                toWaypoint.Location.Altitude = restoLocation.Altitude;
                                toWaypoint.Location.Latitude = restoLocation.Latitude;
                                toWaypoint.Location.Longitude = restoLocation.Longitude;

                                // Create the request.
                                var routeRequest = new RouteRequest();
                                routeRequest.Credentials = new Credentials();
                                routeRequest.Credentials.ApplicationId = BingApiKey;
                                routeRequest.Waypoints = new Waypoint[2];
                                routeRequest.Waypoints[0] = fromWaypoint;
                                routeRequest.Waypoints[1] = toWaypoint;
                                routeRequest.Options = new RouteOptions();
                                routeRequest.Options.RoutePathType = RoutePathType.Points;
                                routeRequest.UserProfile = new UserProfile();
                                routeRequest.UserProfile.DistanceUnit = DistanceUnit.Kilometer;

                                // Execute the request.
                                var routeClient = new RouteServiceClient();
                                var routeResponse = routeClient.CalculateRoute(routeRequest);

                                // Add the result to the result list.
                                if (routeResponse.Result != null && routeResponse.Result.Summary != null)
                                {
                                    var result = new RestaurantResult();
                                    result.Name = resto.Name;
                                    result.Distance = routeResponse.Result.Summary.Distance;
                                    result.TotalSeconds = routeResponse.Result.Summary.TimeInSeconds;

                                    results.Add(result);
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Trace.WriteLine(ex.Message);
                    }
                });

            // Get the top 3 restaurants.
            int i = 1;
            var topRestaurants = results.OrderBy(o => o.TotalSeconds)
                          .Take(3)
                          .Select(o => { o.Index = i++; return o; });

            // Done.
            near.Restaurants.AddRange(topRestaurants);
            return near;
        }
    }
}