﻿using System;
using System.ComponentModel;
using System.Device.Location;
using System.Collections.ObjectModel;

using Examples.Bing.WP7.Restaurants;

namespace Examples.Bing.WP7.Examples
{
    public class RestaurantsViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets the result.
        /// </summary>
        /// <value>The result.</value>
        public NearResult Result { get; set; }

        /// <summary>
        /// Coordinate watcher.
        /// </summary>
        private IGeoPositionWatcher<GeoCoordinate> coordinateWatcher;

        public RestaurantsViewModel()
        {
            WaitForCurrentPosition();
        }

        /// <summary>
        /// Gets near restaurants once the current position was found.
        /// </summary>
        /// <param name="location">The location.</param>
        private void GetNear(Restaurants.Location location)
        {            
            var client = new RestaurantLocatorClient();
            client.GetNearCompleted += new EventHandler<GetNearCompletedEventArgs>(OnGetNearComplete);
            client.GetNearAsync(location);
        }

        /// <summary>
        /// Display near restaurants.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnGetNearComplete(object sender, GetNearCompletedEventArgs e)
        {
            this.Result = e.Result;
            this.Change("Result");
        }

        /// <summary>
        /// Waits for gps device to load.
        /// </summary>
        private void WaitForCurrentPosition()
        {            
            // Simulate coordinate.
            GeoCoordinateEventMock[] events = new GeoCoordinateEventMock[] 
            { 
                new  GeoCoordinateEventMock { Latitude = 39.25, Longitude = -76.58, Time = new TimeSpan(0,0,5) }
            };

            // Start fake coordinate watcher.
            coordinateWatcher = new EventListGeoLocationMock(events);
            coordinateWatcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(OnGWatcherStatusChanged);
            coordinateWatcher.Start();
        }

        /// <summary>
        /// Watcher status changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnGWatcherStatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            if (e.Status == GeoPositionStatus.Ready)
            {
                coordinateWatcher.Stop();

                var location = new Restaurants.Location();
                location.Latitude = coordinateWatcher.Position.Location.Latitude;
                location.Longitude = coordinateWatcher.Position.Location.Longitude;
                GetNear(location);
            }
        }

        private void Change(string property)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(property));
        }
    }
}
