﻿using System;
using System.Linq;
using System.Windows;
using System.ComponentModel;

using Utils.WP7.Bing.BingGeo;

using Microsoft.Phone.Controls;
using Microsoft.Phone.Controls.Maps;

namespace Examples.Bing.WP7
{
    public partial class MainPage : PhoneApplicationPage, INotifyPropertyChanged
    {
        #region Events
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion
        #region Fields and Properties
        private GeocodeLocation location;
        public GeocodeLocation Location
        {
            get { return location; }
            set
            {
                location = value;
                if (PropertyChanged != null)
                    PropertyChanged(this, new PropertyChangedEventArgs("Location"));
            }
        }

        private string error;
        public string Error
        {
            get { return error; }
            set
            {
                error = value;
                if (PropertyChanged != null)
                    PropertyChanged(this, new PropertyChangedEventArgs("Error"));
            }
        }
        #endregion

        public MainPage()
        {
            InitializeComponent();
            DataContext = this;
        }

        private void OnCloud(object sender, RoutedEventArgs e)
        {
            (App.Current as App).RootFrame.Navigate(new Uri("/Examples/RestaurantsView.xaml", UriKind.Relative));
        }

        private void OnGeoRequest(object sender, System.Windows.RoutedEventArgs evt)
        {
            // Reset display.
            Error = String.Empty;
            Location = null;

            // Create the request.
            var geoRequest = new GeocodeRequest();
            geoRequest.Credentials = new Credentials();
            geoRequest.Credentials.ApplicationId = App.BingApiKey;
            geoRequest.Address = new Address();
            geoRequest.Address.CountryRegion = "Belgium";
            geoRequest.Address.PostalTown = "Brussels";

            // Execute the request and display the results.
            var geoClient = new GeocodeServiceClient("BasicHttpBinding_IGeocodeService");
            geoClient.GeocodeAsync(geoRequest);
            geoClient.GeocodeCompleted += (s, e) => 
            {
                if (e.Result != null && e.Result.Results.Any(o => o.Locations != null && o.Locations.Any()))
                    Location = e.Result.Results.FirstOrDefault().Locations.FirstOrDefault();
                else if (e.Error != null)
                    Error = e.Error.Message;
                else
                    Error = "No results or locations found.";
            };
        }

        private void OnCalculateRoute(object sender, RoutedEventArgs e)
        {
            (App.Current as App).RootFrame.Navigate(new Uri("/Examples/RouteView.xaml", UriKind.Relative));
        }

    }
}